#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include <config.h>

#include "Lom.h"
#include "Render.h"

static GtkWidget *child;
GtkWidget *gameview;

extern void AboutLOM(GtkWidget *w, gpointer data);
extern void LoadSavedGame(GtkWidget *w, gpointer data);
extern void BeginNewGame(GtkWidget *w, gpointer data);
extern void SaveGame(GtkWidget *w, gpointer data);
extern void ExitGame(GtkWidget *w, gpointer data);


static GnomeUIInfo help_menu[]={
	GNOMEUIINFO_HELP("lom"),
	{GNOME_APP_UI_ITEM, "About...", "About this application",
	AboutLOM, NULL, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL },
	GNOMEUIINFO_END
};

static GnomeUIInfo file_menu[] = {
	{GNOME_APP_UI_ITEM, 
		"New Game", "Begin a new game", 
		BeginNewGame, NULL, NULL, 
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,
		'n', GDK_CONTROL_MASK, NULL},
	{GNOME_APP_UI_SEPARATOR},
	{GNOME_APP_UI_ITEM, 
		"Load Game", "Restore a saved game",
		LoadSavedGame, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
		'l', GDK_CONTROL_MASK, NULL},
	{GNOME_APP_UI_ITEM, 
		"Save...", "Save game to disk", 
		SaveGame, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
		's', GDK_CONTROL_MASK, NULL },
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM,
		"Exit", "Quit the game without saving",
		ExitGame, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 
		'q', GDK_CONTROL_MASK, NULL },
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
	{GNOME_APP_UI_SUBTREE, "File", NULL, file_menu, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
#define MENU_HELP_POS 1
	{GNOME_APP_UI_SUBTREE, "Help", NULL, help_menu, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
	{GNOME_APP_UI_ENDOFINFO}
};

char *ColourNames[8]={
	"#000000","#0000FF","#FF0000","#FF00FF",
        "#00FF00","#00FFFF","#FFFF00","#FFFFFF"
};
                   
GdkGC *WhiteInk;
GdkGC *BlueInk;
GdkGC *Colours[8];


/*
 *	Menu call backs
 */
 
void AboutLOM(GtkWidget *w, gpointer data) {}

void LoadSavedGame(GtkWidget *w, gpointer data) {
}

void BeginNewGame(GtkWidget *w, gpointer data) {
	MainGameLoopInstance(0);
}

void SaveGame(GtkWidget *w, gpointer data) {
}

void ExitGame(GtkWidget *w, gpointer data) {}

/*
 *	These handle the title panel events
 */

#if 0
static int DrawEvent(GtkWidget *g, void *private, XEvent *event)
{
	switch(event->type)
	{
		case Expose:
			/*
			 *	Just for feel
			 */
			XFillRectangle(OurDisplay, DrawingZone->XObject, FillGC[1], 2,2, 512, 256);
			XFillRectangle(OurDisplay, DrawingZone->XObject, FillGC[7], 2, 260, 512, 138);
			DrawFeature(1,0,100,320);
			/*
			 *	The final version will copy the landscaping
			 *	pixmap buffer to screen.
			 */
			/*XCopyArea(OurDisplay, title_map, g->XObject,Shadow,0,0,256,192,4,4);*/
			break;	
	}
	return 0;
}

#endif

/*
 *	Keyboard logic
 */
 
static void KeyboardEvent(GtkWidget *w, GdkEvent *event)
{
	if(event->type==GDK_KEY_PRESS)
	{
		printf("Key %d\n", event->key.keyval);
		switch(event->key.keyval)
		{
			case GDK_Left:
				Keyboard('-');
				break;
			case GDK_Right:
				Keyboard('=');
				break;
			case GDK_Up:
				Keyboard('q');
				break;
			default:
				if(event->key.keyval<255)
					Keyboard(event->key.keyval);
		}
	}
}
	

void CreateGameWindow(void) 
{
	int xo;
	int i;
	
	GtkWidget *vbox;

	child=gnome_app_new("lom", "Lords Of Midnight");
	gtk_window_set_wmclass(GTK_WINDOW(child), "lom","Lords Of Midnight");

	gnome_app_create_menus(GNOME_APP(child), main_menu);
	gtk_menu_item_right_justify(GTK_MENU_ITEM(main_menu[MENU_HELP_POS].widget));

	gameview = gnome_pixmap_new_from_rgb_d_at_size("\0\0\0", "\0\0\0", 1, 1, 512, 384);

	gtk_widget_set_events(gameview, GDK_ALL_EVENTS_MASK);
	GTK_WIDGET_SET_FLAGS(gameview, GTK_CAN_FOCUS);
	gtk_widget_grab_focus(gameview);
	gtk_signal_connect(GTK_OBJECT(child), "key_press_event",
            (GtkSignalFunc)KeyboardEvent, 0);  
//	gameview = gtk_drawing_area_new();
//	gtk_drawing_area_size(GTK_DRAWING_AREA(gameview), 512, 384);

	vbox=gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gameview, TRUE, TRUE, 0);
	gnome_app_set_contents(GNOME_APP(child), vbox);
	
	/*
	 *	Now load the colour table
	 */
	 
	gtk_widget_realize(gameview);
	
	for(i=0;i<8;i++)
	{	
		GdkColor c;
		gdk_color_parse(ColourNames[i], &c);
		gdk_color_alloc(gdk_window_get_colormap(gameview->window), &c);
		Colours[i]=gdk_gc_new(gameview->window);
		gdk_gc_set_foreground(Colours[i], &c);
	}
	
	BlueInk=Colours[1];
	WhiteInk=Colours[7];
	gtk_widget_show_all(child);
}


/*
 *	Startup panel top level messages
 */
 
static void TopEvent(GtkWidget *w, gpointer *private)
{
	gtk_widget_destroy(child);
	CreateGameWindow();
}

/*
 *	Main Entry Point
 */

void main(int argc, char *argv[])
{
	int x,y;
	char *s;
	GtkWidget *pixmap = NULL;
		
	argp_program_version = VERSION;
	
	gnome_init("Lords Of Midnight", 0, argc, argv, 0, 0);
	
	s=gnome_pixmap_file("lords-title.xpm");
	
	if(s)
	{
		pixmap=gnome_pixmap_new_from_file(s);
		g_free(s);
	}
	
	if(pixmap==NULL)
		pixmap = gtk_label_new("Logo file missing");
		
	
	child=gnome_dialog_new("Lords Of Midnight", GNOME_STOCK_BUTTON_OK, NULL);
	gnome_dialog_set_close(GNOME_DIALOG(child), TRUE);
	
	gtk_signal_connect(GTK_OBJECT(child), "close",
		GTK_SIGNAL_FUNC(TopEvent), NULL);
		
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(child)->vbox), pixmap,
		FALSE, FALSE, GNOME_PAD);
	
	gtk_window_position(GTK_WINDOW(child), GTK_WIN_POS_CENTER);
	gtk_widget_show_all(child);
	gtk_main();
}
