#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include "Lom.h"
#include "Render.h"

extern GdkGC *BlueInk;
extern GdkGC *WhiteInk;

/*
 *	Terrain Rendering Functions
 */

static void DrawLine(GdkDrawable *b, int y, int xs,int xe, int plot)
{
	xe++;
	if(xs<0)
		xs=0;
		
	if(plot)
	{
		gdk_draw_line(b, BlueInk, xs,y, xe,y);
		gdk_draw_line(b, BlueInk, xs,y+1, xe,y+1);
	}
	else
	{
		gdk_draw_line(b, WhiteInk, xs,y, xe,y);
		gdk_draw_line(b, WhiteInk, xs,y+1, xe,y+1);
	}
}

static void Pset(GdkDrawable *b , int y, int x, int plot)
{
	gdk_draw_point(b, plot?WhiteInk:BlueInk, x,y);
	gdk_draw_point(b, plot?WhiteInk:BlueInk, x+1,y);
	gdk_draw_point(b, plot?WhiteInk:BlueInk, x+1,y+1);
	gdk_draw_point(b, plot?WhiteInk:BlueInk, x,y+1);
}

static void Preset(GdkDrawable *b, int y, int x)
{
	gdk_draw_point(b, BlueInk, x,y);
	gdk_draw_point(b, BlueInk, x+1,y);
	gdk_draw_point(b, BlueInk, x+1,y+1);
	gdk_draw_point(b, BlueInk, x,y+1);
}


static void SetInks(int t)
{
	if(t)
	{
		BlueInk=Colours[1];
		WhiteInk=Colours[7];
	}
	else
	{
		BlueInk=Colours[1];
		WhiteInk=Colours[0];
	}
}

/*
 *	Render a feature onto a Gnome Pixmap
 */	
 
void DrawFeature(GtkWidget *img, int time, int feature, int size, int xoff, int yoff)
{
	GdkDrawable *target;
	char *bp=TerrainImages[feature<<3|size];
	int height=*bp++;
	int anotherbit;
	int xsave=xoff;
	int yp=0;
	
	SetInks(time);
	
	target = img->window;

	do
	{
		do
		{
			int ploton=1;
			int ops;
			int start,end;
			
			anotherbit=0;
			
			if(*bp&0x80)
				anotherbit=1;
			if(*bp&0x40)
				ploton=0;
			ops=*bp++&0x3F;		/* Op count */
			start=*bp++;
			end=*bp++;
			DrawLine(target, yoff-2*yp,2*start+xoff,2*end+xoff,ploton);
			Pset(target, yoff-2*yp,2*start+xoff,ploton);
			Pset(target, yoff-2*yp,2*end+xoff,ploton);
			ploton^=1;
			while(ops>0)
			{
				if(!(*bp&0x80))	/* Plot ? */
					Preset(target, yoff-2*yp,xoff+2*(*bp&0x7F));
				else
				{
					start=*bp++&0x7F;
					end=*bp;
					DrawLine(target, yoff-2*yp,xoff+2*start,xoff+2*end,ploton);
				}
				bp++;
				ops--;
			}
		}
		while(anotherbit);
		yp++;
		xoff=xsave;
	}
	while(--height>0);
}

void DrawHorizon(GtkWidget *w, int time)
{
	GdkDrawable *target = w->window;

	SetInks(time);
		
	gdk_draw_rectangle(target, BlueInk, 1, 0, 128, 512, 128);
	gdk_draw_rectangle(target, WhiteInk, 1, 0, 256, 512, 128);
}
